
#include <Wire.h>
#include <LiquidCrystal_I2C.h>
#include <Servo.h>
#include <EasyButton.h>

LiquidCrystal_I2C lcd(0x27, 16, 2);

#define LCD_START_COLUMN 1
#define LCD_CHAR_DELTA 2

// Create a new servo object:
Servo myservo;

// Define the servo pin:
#define servoPin 12

#define BUTTON_PIN_1 10
#define BUTTON_PIN_2 9
#define BUTTON_PIN_3 8
#define BUTTON_PIN_4 7
#define BUTTON_PIN_5 6
#define BUTTON_PIN_6 5
#define BUTTON_PIN_7 4
#define BUTTON_PIN_8 3
#define BUTTON_PIN_9 2

EasyButton button1(BUTTON_PIN_1);
EasyButton button2(BUTTON_PIN_2);
EasyButton button3(BUTTON_PIN_3);
EasyButton button4(BUTTON_PIN_4);
EasyButton button5(BUTTON_PIN_5);
EasyButton button6(BUTTON_PIN_6);
EasyButton button7(BUTTON_PIN_7);
EasyButton button7(BUTTON_PIN_8);
EasyButton button7(BUTTON_PIN_9);

#define NUM_BUTTONS 9
#define NUM_OPTIONS 4

const char optionChars[NUM_BUTTONS][NUM_OPTIONS] = {
  "AXBC", 
  "ABXC", 
  "ABCX",
  "XABC", 
  "AXBC", 
  "ABXC", 
  "ABCX", 
  "XABC", 
  "AXBC"
};

byte currentIndex[NUM_BUTTONS] = {0, 0, 0, 0, 0, 0, 0, 0, 0};
const byte solutionIndex[NUM_BUTTONS] = {1, 2, 3, 0, 1, 2, 3, 0, 1};

bool isLockOpen = false;

int isAdminMode = 0;

#define SERVO_LOCK_OPEN_POS 700
#define SERVO_LOCK_CLOSE_POS 1580
#define SERVO_LOCK_MAX 2400

int servoPosition = 1000;

void setup() {
  Serial.begin (9600); //Start serial com so we could print the value on the serial monitor
  Serial.println("start");

  initLcd();

  initServo();

  initButtons();

}

void initLcd() {
  Serial.println("lcd init");
  lcd.init();

  Serial.println("backlight");
  lcd.backlight();
  
  delay(100);

  Serial.println("lcd write");
  lcd.setCursor(0, 0);
  lcd.print("                ");

  initView();  
  
  Serial.println("lcd after write");  
}

void initServo() {
  // Attach the Servo variable to a pin:
  myservo.attach(servoPin);

  setLockStatus(false);
}


void initButtons() {
  // Initialize the buttons
  button1.begin();
  button2.begin();
  button3.begin();
  button4.begin();
  button5.begin();
  button6.begin();
  button7.begin();
  button8.begin();
  button9.begin();

  // Attach callbacks
  button1.onPressed(onPressedFirstButton);
  button2.onPressed(onPressed2);
  button3.onPressed(onPressed3);
  button4.onPressed(onPressed4);
  button5.onPressed(onPressed5);
  button6.onPressed(onPressed6);
  button7.onPressed(onPressed6);
  button8.onPressed(onPressed6);
  button9.onPressed(onPressedLastButton);

  // ability to enter Admin Mode
  button1.onPressedFor(5000, onPressedForDurationFirstButton);
  button1.onSequence(5, 2000, onSequenceMatchedFirstButton);
  button9.onPressedFor(5000, onPressedForDurationLastButton);
  button9.onSequence(5, 2000, onSequenceMatchedLastButton);
}

void loop() { 
    button1.read();
    button2.read();
    button3.read();
    button4.read();
    button5.read();
    button6.read();
    button7.read();
    button8.read();
    button9.read();
}

void setLockStatus(bool setIsLockOpened)
{
  if (setIsLockOpened) {
    myservo.writeMicroseconds(SERVO_LOCK_OPEN_POS);
  } else {
    myservo.writeMicroseconds(SERVO_LOCK_CLOSE_POS);    
  }
}

void onButtonChanged(byte buttonIndex) {
  if (!isLockOpen) {
    updateModel(buttonIndex);
    updateView(buttonIndex);
    
    if (checkIfSolved()) {
      isLockOpen = true;
      setLockStatus(true);
    }
  }
}

void updateModel(byte buttonIndex) {
  if (currentIndex[buttonIndex] >= NUM_OPTIONS - 1) {
    currentIndex[buttonIndex] = 0;
  } else {
    currentIndex[buttonIndex]++;    
  }
}

void initView() {
  lcd.setCursor(0, 1);
  char lcdOutput[] = "                ";
  char charTemp;
  for (byte i=0; i < NUM_BUTTONS; i++) {
    charTemp = optionChars[i][currentIndex[i]];
    // output every other character on the LCD
    lcdOutput[i*LCD_CHAR_DELTA + LCD_START_COLUMN] = charTemp;
  }
  lcd.print(lcdOutput);
}

void updateView(byte buttonIndex) {
  char charTemp = optionChars[buttonIndex][currentIndex[buttonIndex]];
  Serial.println(charTemp);  
  
  // output every other character on the LCD
  lcd.setCursor(buttonIndex*LCD_CHAR_DELTA + LCD_START_COLUMN, 1);
  lcd.print(charTemp);
}

void setViewAsAdminMode() {
  lcd.setCursor(0, 0);
  lcd.print("   Admin Mode   ");
  lcd.setCursor(0, 1);
  lcd.print("                ");
}

bool checkIfSolved() {
  bool isSolved = true;
  for (byte i = 0; i < NUM_BUTTONS; i++)
  {
    if (currentIndex[i] != solutionIndex[i])
    {
      isSolved = false;
      // exit for loop
      i = NUM_BUTTONS;
    }
  }
  return isSolved;
}


void onPressedFirstButton() {
  if (isAdminMode == 1)
  {
    Serial.println("Opening lock");
    setLockStatus(true);    
  }
  else
  {
    Serial.println("Button 1");
    onButtonChanged(0);    
  }
}

void onPressed2() {
  if (isAdminMode != 1)
  {
    Serial.println("Button 2");
    onButtonChanged(1);    
  }
  else if (isAdminMode == 1)
  {
    servoPosition -= 10;
    Serial.println(servoPosition);
    myservo.writeMicroseconds(servoPosition);
  }
}

void onPressed3() {
  if (isAdminMode != 1)
  {
    Serial.println("Button 3");
    onButtonChanged(2);    
  }
}

void onPressed4() {
  if (isAdminMode != 1)
  {
    Serial.println("Button 4");
    onButtonChanged(3);
  }
}

void onPressed5() {
  if (isAdminMode != 1)
  {
    Serial.println("Button 5");
    onButtonChanged(4); 
  }
}

void onPressed6() {
  if (isAdminMode != 1)
  {
    Serial.println("Button 6");
    onButtonChanged(5);
  }
  else if (isAdminMode == 1)
  {
    servoPosition += 10;
    Serial.println(servoPosition);
    myservo.writeMicroseconds(servoPosition);
  }
}

void onPressed7() {
  if (isAdminMode != 1)
  {
    Serial.println("Button 7");
    onButtonChanged(6);
  }
  else if (isAdminMode == 1)
  {
    servoPosition += 10;
    Serial.println(servoPosition);
    myservo.writeMicroseconds(servoPosition);
  }
}

void onPressed8() {
  if (isAdminMode != 1)
  {
    Serial.println("Button 8");
    onButtonChanged(7);
  }
  else if (isAdminMode == 1)
  {
    servoPosition += 10;
    Serial.println(servoPosition);
    myservo.writeMicroseconds(servoPosition);
  }
}

void onPressedLastButton() {
  if (isAdminMode == 1)
  {
    Serial.println("Closing lock");
    setLockStatus(false);
  }
  else
  {
    Serial.println("Button 9");
    onButtonChanged(8);
  }
}

void onPressedForDurationFirstButton() {
  if (isAdminMode == 1)
  {
    servoPosition = SERVO_LOCK_OPEN_POS;
    Serial.println(servoPosition);
    myservo.writeMicroseconds(servoPosition);
  }
  else
  {
    Serial.println("First button pressed for 5 seconds");
    if (isAdminMode == 0) isAdminMode = -1;
  }  
}

void onSequenceMatchedFirstButton() {
  Serial.println("First button pressed 5 times in 2 seconds");
  if (isAdminMode == -1) isAdminMode = -2;
  else isAdminMode = 0;
}

void onPressedForDurationLastButton() {
  Serial.println("Last button pressed for 5 seconds");
  if (isAdminMode == 1)
  {
    servoPosition = SERVO_LOCK_MAX;
    Serial.println(servoPosition);
    myservo.writeMicroseconds(servoPosition);
  } 
  else 
  {  
    if (isAdminMode == -2) isAdminMode = -3;
    else isAdminMode = 0;
  }

}

void onSequenceMatchedLastButton() {
  Serial.println("Last button pressed 5 times in 2 seconds");
  if (isAdminMode == -3) 
  {
    isAdminMode = 1;
    Serial.println("Admin Mode Enabled!");  
    setViewAsAdminMode();
  }
}



